<?php
/**
 * securepayxml.php
 *
 * Implements the SecurePay XML API payment module for Oscommerce
 *
 * Contains the securepayxml class, which handles SecurePay XML API credit-card transactions via the securepay_xml_transaction class (securepay_xml_api.php).
 * The bulk of this class is a wrapper for securepay_xml_transaction, to interface with the Oscommerce way of doing things. It also sets up some text, menus and database entries.
 *
 * Supports the following kind of transactions:
 *	Standard Credit
 *	Preauthorisation
 *
 * Preauthorised transactions will need to be completed via the SecurePay MMF, as Oscommerce does not appear to support administrative order hooks (which permit Advice/Refund/Reverse in Zen Cart).
 *
 * @author Andrew Dubbeld (support@securepay.com.au)
 * @license http://www.oscommerce.com/license/2_0.txt GNU Public License V2.0
 * @version $Id: securepayxml.php 7341 2009-10-14 06:29:30Z $
 * @date 14-Oct-2009
 */
 
require_once(DIR_FS_CATALOG . DIR_WS_MODULES . 'payment/securepay_xml_api.php');

if (!defined('DB_PREFIX'))
{
	define('DB_PREFIX','');
}
if (!defined('TABLE_SECUREPAYXML'))
{
	define('TABLE_SECUREPAYXML', DB_PREFIX . 'securepayxml');
}

class securepayxml
{
	public $payment_status, $auth_code, $transaction_id;
	private $_logDir = DIR_FS_CACHE;
	
	private $mode = SECUREPAY_GATEWAY_MODE_TEST;
	
	public $title,$code, $description, $enabled;

	function securepayxml()
	{
		global $order, $messageStack;
		
		$this->code = 'securepayxml';
		$this->enabled = ((MODULE_PAYMENT_SECUREPAYXML_STATUS == 'True') ? true : false); // Whether the module is installed or not
		
		if ($_GET['set']!='payment' && IS_ADMIN_FLAG == 'IS_ADMIN_FLAG')
		{
			$this->title = MODULE_PAYMENT_SECUREPAYXML_TEXT_CATALOG_TITLE; // Payment module title in Catalog
		}
		else
		{
			$this->title = MODULE_PAYMENT_SECUREPAYXML_TEXT_ADMIN_TITLE; // Payment module title in Admin
			if ($this->enabled && !function_exists('curl_init')) 
			{
				$messageStack->add_session(MODULE_PAYMENT_SECUREPAYXML_TEXT_ERROR_CURL_NOT_FOUND, 'error');
			}
		}
		
		$this->description = MODULE_PAYMENT_SECUREPAYXML_TEXT_DESCRIPTION;	// Descriptive Info about module in Admin
		$this->sort_order = MODULE_PAYMENT_SECUREPAYXML_SORT_ORDER; // Sort Order of this payment option on the customer payment page
		$this->form_action_url = FILENAME_CHECKOUT_PROCESS; // Page to go to upon submitting page info

		$this->order_status = (int)DEFAULT_ORDERS_STATUS_ID;

		$this->mode = SECUREPAY_GATEWAY_MODE_TEST;
		if (MODULE_PAYMENT_SECUREPAYXML_TEST == "Live")
		{
			$this->mode = SECUREPAY_GATEWAY_MODE_LIVE;
		}
		
		if ((int)MODULE_PAYMENT_SECUREPAYXML_ORDER_STATUS_ID > 0)
		{
			$this->order_status = MODULE_PAYMENT_SECUREPAYXML_ORDER_STATUS_ID;
		}
		
		if (MODULE_PAYMENT_SECUREPAYXML_MODE == MODULE_PAYMENT_SECUREPAYXML_MODE_PREAUTH && (int)MODULE_PAYMENT_SECUREPAYXML_PREAUTH_ORDER_STATUS_ID > 0)
		{
			$this->order_status = MODULE_PAYMENT_SECUREPAYXML_PREAUTH_ORDER_STATUS_ID;
		}
		
		$this->zone = (int)MODULE_PAYMENT_SECUREPAYXML_ZONE;
		
		if (is_object($order))
		{
			$this->update_status();
		}
		
		$this->code_debug = (MODULE_PAYMENT_SECUREPAYXML_CODE_DEBUG=='debug') ? true : false;

		// set error messages if misconfigured
		if (MODULE_PAYMENT_SECUREPAYXML_STATUS == 'True')
		{
			if (MODULE_PAYMENT_SECUREPAYXML_MERCHANTID == '')
			{
				$this->title .= MODULE_PAYMENT_SECUREPAYXML_TEXT_NOT_CONFIGURED;
			}
			elseif (MODULE_PAYMENT_SECUREPAYXML_TEST != 'Live')
			{
				$this->title .= MODULE_PAYMENT_SECUREPAYXML_TEXT_TEST_MODE;
			}
		}
	}

	/**
	 * update status
	 *
	 * disables module if customer is incorrect zone
	 */
	function update_status()
	{
		global $order;

		if ( $this->enabled && $this->zone > 0 )
		{
			$check_flag = false;
			$sql = "SELECT zone_id
					FROM " . TABLE_ZONES_TO_GEO_ZONES . "
					WHERE geo_zone_id = " . htmlentities($this->zone) . "
					AND zone_country_id = " . htmlentities($order->billing['country']['id']) . "
					ORDER BY zone_id";
			$check = tep_db_query($sql);
			while (!$check->EOF)
			{
				if ($check->fields['zone_id'] < 1)
				{
					$check_flag = true;
					break;
				}
				elseif ($check->fields['zone_id'] == $order->billing['zone_id'])
				{
					$check_flag = true;
					break;
				}
				$check->MoveNext();
			}

			if (!$check_flag)
			{
				$this->enabled = false;
			}
		}
	}

	/**
	 * javascript_validation
	 *
	 * return value is included in the payment form.
	 * First stage of form validation
	 */
	function javascript_validation()
	{
		$js = 	'	if (payment_value == "' . $this->code . '") {' . "\n" .
				'		var cc_owner = document.checkout_payment.securepayxml_cc_owner.value;' . "\n" .
				'		var cc_number = document.checkout_payment.securepayxml_cc_number.value;' . "\n" .
				'		var cc_cvv = document.checkout_payment.securepayxml_cc_cvv.value;' . "\n" .
				'		if (cc_owner == "" || cc_owner.length < ' . CC_OWNER_MIN_LENGTH . ') {' . "\n" .
				'			error_message = error_message + "' . MODULE_PAYMENT_SECUREPAYXML_TEXT_JS_CC_OWNER . '";' . "\n" .
				'			error = 1;' . "\n" .
				'		}' . "\n" .
				'		if (cc_number == "" || cc_number.length < ' . CC_NUMBER_MIN_LENGTH . ') {' . "\n" .
				'			error_message = error_message + "' . MODULE_PAYMENT_SECUREPAYXML_TEXT_JS_CC_NUMBER . '";' . "\n" .
				'			error = 1;' . "\n" .
				'		}' . "\n" .
				'		if (cc_cvv == "" || cc_cvv.length < "3") {' . "\n".
				'			error_message = error_message + "' . MODULE_PAYMENT_SECUREPAYXML_TEXT_JS_CC_CVV . '";' . "\n" .
				'			error = 1;' . "\n" .
				'		}' . "\n" .
				'	}' . "\n";

		return $js;
	}

	/**
	 * selection
	 *
	 * Generates the payment form, as part of the checkout process.
	 */
	function selection()
	{
		global $order;

		for ($i=1; $i<13; $i++)
		{
			$expires_month[] = array('id' => sprintf('%02d', $i), 'text' => strftime('%B - (%m)',mktime(0,0,0,$i,1,2000)));
		}

		$today = getdate();
		for ($i=$today['year']; $i < $today['year']+10; $i++)
		{
			$expires_year[] = array('id' => strftime('%y',mktime(0,0,0,1,1,$i)), 'text' => strftime('%Y',mktime(0,0,0,1,1,$i)));
		}

		$onFocus = ' onfocus="methodSelect(\'pmt-' . $this->code . '\')"';

		$selection = array(	
			'id' => $this->code,
			'module' => $this->title,
			'fields' => array(
				array(
					'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_CREDIT_CARD_OWNER,
					'field' => tep_draw_input_field('securepayxml_cc_owner', $order->billing['firstname'] . ' ' . $order->billing['lastname'], 'id="'.$this->code.'-cc-owner"'. $onFocus),
					'tag' => $this->code.'-cc-owner'
				),
				array(
					'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_CREDIT_CARD_NUMBER,
					'field' => tep_draw_input_field('securepayxml_cc_number', $ccnum, ' autocomplete="off" id="'.$this->code.'-cc-number"' . $onFocus),
					'tag' => $this->code.'-cc-number'
				),
				array(
					'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_CREDIT_CARD_EXPIRES,
					'field' => tep_draw_pull_down_menu('securepayxml_cc_expires_month', $expires_month, '', 'id="'.$this->code.'-cc-expires-month"' . $onFocus) . '&nbsp;' . tep_draw_pull_down_menu('securepayxml_cc_expires_year', $expires_year, '', 'id="'.$this->code.'-cc-expires-year"' . $onFocus),
					'tag' => $this->code.'-cc-expires-month'
				),
				array(
					'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_CVV,
					'field' => tep_draw_input_field('securepayxml_cc_cvv', '', 'size="4" maxlength="4"'. ' autocomplete="off" id="'.$this->code.'-cc-cvv"' . $onFocus) . ' ' . '<a href="javascript:popupWindow(\'' . tep_href_link(FILENAME_POPUP_CVV_HELP) . '\')">' . MODULE_PAYMENT_SECUREPAYXML_TEXT_POPUP_CVV_LINK . '</a>',
					'tag' => $this->code.'-cc-cvv'
				)
			)
		);

		return $selection;
	}

	/**
	 * pre_confirmation_check
	 *
	 * Secondary credit-card detail check (back-end)
	 * Third (redundant) check is performed in securepay_xml_transaction class
	 */
	function pre_confirmation_check()
	{
		global $messageStack;

		include(DIR_WS_CLASSES . 'cc_validation.php');

		$cc_validation = new cc_validation();
		$result = $cc_validation->validate($_POST['securepayxml_cc_number'], $_POST['securepayxml_cc_expires_month'], $_POST['securepayxml_cc_expires_year']);
		$error = '';
		
		switch ($result)
		{
			case -1:
				$error = sprintf(TEXT_CCVAL_ERROR_UNKNOWN_CARD, substr($cc_validation->cc_number, 0, 4));
				break;
			case -2:
			case -3:
			case -4:
				$error = TEXT_CCVAL_ERROR_INVALID_DATE;
				break;
			case false:
				$error = TEXT_CCVAL_ERROR_INVALID_NUMBER;
				break;
		}
		
		// if no error, continue with validated data:
		$this->cc_card_type = $cc_validation->cc_type;
		$this->cc_card_number = $cc_validation->cc_number;
		$this->cc_expiry_month = $cc_validation->cc_expiry_month;
		$this->cc_expiry_year = $cc_validation->cc_expiry_year;
	}

	/**
	 * confirmation
	 *
	 * Displayed on checkout_confirm. Shows payment/credit info
	 */
	function confirmation()
	{
		$confirmation = array(
			'title' => $this->title . ': ' . $this->cc_card_type,
			'fields' => array(
				array(
					'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_CREDIT_CARD_OWNER,
					'field' => htmlentities($_POST['securepayxml_cc_owner'])
				),
				array(
					'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_CREDIT_CARD_NUMBER,
					'field' => str_repeat('X', (strlen($this->cc_card_number) - 4)) . substr($this->cc_card_number, -4)
				),
				array(
					'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_CREDIT_CARD_EXPIRES,
					'field' => strftime('%B, %Y', mktime(0,0,0,htmlentities($_POST['securepayxml_cc_expires_month']), 1, '20' . htmlentities($_POST['securepayxml_cc_expires_year'])))
				)
			)
		);

		return $confirmation;
	}
	
	/**
	 * process_button
	 *
	 * Passes the payment details to before_process via post
	 *
	 * Not sure why this is the preferred method of passing CC details, but it is common to all payment modules, and over SSL it isn't too terrible.
	 *
	 * @return string process_button_string - Included in checkout_confirmation form
	 */
	function process_button()
	{
		global $osCsid;
		
		// These are hidden fields on the checkout confirmation page
		$process_button_string = 
			tep_draw_hidden_field('cc_owner', $_POST['securepayxml_cc_owner']) .
			tep_draw_hidden_field('cc_expires', $this->cc_expiry_month . substr($this->cc_expiry_year, -2)) .
			tep_draw_hidden_field('cc_expires_month', $this->cc_expiry_month) .
			tep_draw_hidden_field('cc_expires_year', substr($this->cc_expiry_year, -2)) .
			tep_draw_hidden_field('cc_type', $this->cc_card_type) .
			tep_draw_hidden_field('cc_number', $this->cc_card_number) .
			tep_draw_hidden_field('cc_cvv', $_POST['securepayxml_cc_cvv']) .
			tep_draw_hidden_field(tep_session_name(), tep_session_id());
			
		return $process_button_string;
	}
	
	/**
	 * before_process
	 *
	 * Prepares and submits payment info to SecurePay via the securepay_xml_transaction class
	 * Also populates securepayxml table with transaction details after success.
	 */
	function before_process()
	{
		global $order, $messageStack, $_POST, $_SESSION;
		$preauth = "";
		$txnid = "";
		
		$sxml = new securepay_xml_transaction($this->mode,MODULE_PAYMENT_SECUREPAYXML_MERCHANTID,MODULE_PAYMENT_SECUREPAYXML_MERCHANTPASS);
		
		$customer_id = $_SESSION['customer_id'];
		
		$amount = $order->info['total'];
		
		$last_order_id = tep_db_fetch_array(tep_db_query("select orders_id from ".TABLE_ORDERS." order by orders.orders_id desc limit 1"));
		$new_order_id = $last_order_id['orders_id'];
		$oid = ($new_order_id + 1);
		
		$type = SECUREPAY_TXN_STANDARD;
		if(MODULE_PAYMENT_SECUREPAYXML_MODE == MODULE_PAYMENT_SECUREPAYXML_MODE_PREAUTH)
		{
			$type = SECUREPAY_TXN_PREAUTH;
		}
		
		// CARD INFO
		$cc_number = $_POST['cc_number'];
		$cc_month = $_POST['cc_expires_month'];
		$cc_year = $_POST['cc_expires_year'];
		$cvv = $_POST['cc_cvv'];
		
		if($type == SECUREPAY_TXN_PREAUTH)
		{
			$result = $sxml->processCreditPreauth($amount,$oid,$cc_number,$cc_month,$cc_year,$cvv);
		}
		else
		{
			$result = $sxml->processCreditStandard($amount,$oid,$cc_number,$cc_month,$cc_year,$cvv);
		}
		
		$txnResultCodeText = $sxml->getErrorString();
		$approved = strtoupper($sxml->getResultByKeyName('approved'))=='YES'?true:false;
		$status = $sxml->getResultByKeyName('responseCode');
		$this->transaction_id = $result;
		
		if(($status != '000' && $status != '008') || $approved==false)
		{
			//Gateway response error
			$this->_log($txnResultCodeText." on transaction by " . htmlentities($customer_id) . " for $" . $amount);
			$messageStack->add('checkout_payment','Credit Card declined', 'error');
			
			$error = ((MODULE_PAYMENT_SECUREPAYXML_ERROR_DISPLAY=='Yes')?$txnResultCodeText:MODULE_PAYMENT_SECUREPAYXML_ERROR_TEXT);
			
			tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . urlencode($error), 'SSL', true, false));
		}
		
		//Store Transaction history in Database
		$sql_data_array = array(
			'customer_id' => $customer_id, 	//Local: 	customer's id
			'banktxnid' => $result, 		//Gateway: 	transaction id
			'oid' => $oid, 					//Local: 	order id/reference
			'response_code' => $status, 	//Gateway: 	response code (000 - 999)
			'time' => $sxml->getGMTTimeStamp(), //Local: 	timestamp
			'total' => $amount, 			//Local: 	Total $
			'txntype' => $type, 			//Local: 	Txn Type. See securepay_xml_api.php
		);
		
		tep_db_perform(TABLE_SECUREPAYXML, $sql_data_array);
		
		return true;
	}
	
	/**
	 * after_process
	 *
	 * Updates order record with invoice details + new status
	 */
	function after_process()
	{
		global $insert_id;
		$comments = (MODULE_PAYMENT_SECUREPAYXML_MODE == MODULE_PAYMENT_SECUREPAYXML_MODE_PREAUTH ? MODULE_PAYMENT_SECUREPAYXML_MODE_PREAUTH_DESC : MODULE_PAYMENT_SECUREPAYXML_MODE_STANDARD_DESC);

		switch (MODULE_PAYMENT_SECUREPAYXML_TEST)
		{
			case "Live": $comments .= ''; break;
			case "Test": $comments .= ' (Test-mode)'; break;
		}
		
		tep_db_query("insert into " . TABLE_ORDERS_STATUS_HISTORY . " (comments, orders_id, orders_status_id, date_added) values ('Credit Card payment. " . $comments . " " . $this->cc_card_type . " Transaction ID: " . $this->transaction_id . "' , '". (int)$insert_id . "','" . $this->order_status . "', now() )");
		return false;
	}
	
	function after_order_create($zf_order_id)
	{
		return;
	}
	
	function get_error()
	{
		$error = array(
			'title' => MODULE_PAYMENT_SECUREPAYXML_TEXT_ERROR,
			'error' => stripslashes(urldecode($_GET['error']))
		);
		return $error;
	}

	/**
	 * Tells the payment module selector (in admin and checkout) whether to list the module.
	 */
    function check()
	{
		if (!isset($this->_check))
		{
			$check_query = tep_db_fetch_array(tep_db_query("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_SECUREPAYXML_STATUS'"));
			$this->_check = ($check_query['configuration_value'] == 'True'?1:0);
		}
		
		return $this->_check;
    }

	function install()
	{
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable Securepayxml Module', 'MODULE_PAYMENT_SECUREPAYXML_STATUS', 'True', 'Do you want to accept SecurePay credit card payments?', '6', 121, 'tep_cfg_select_option(array(\'True\', \'False\'), ', now());");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Merchant ID', 'MODULE_PAYMENT_SECUREPAYXML_MERCHANTID', '', 'Enter your SecurePay Merchant ID.', '6', 121, now());");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Merchant Password', 'MODULE_PAYMENT_SECUREPAYXML_MERCHANTPASS', '', 'Enter your SecurePay Merchant Password.', '6', 121, now());");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Gateway Mode', 'MODULE_PAYMENT_SECUREPAYXML_TEST', 'Test', '<strong>Live:</strong> Use this setting for live stores. Funds will be transferred from the customer\'s account into your merchant account as expected.<br /><strong>Test:</strong> Use this setting to test the module without transferring funds.', '6', 121, 'tep_cfg_select_option(array(\'Live\', \'Test\'), ', now());");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Transaction Type', 'MODULE_PAYMENT_SECUREPAYXML_MODE', 'Standard Payment', 'Do you want submitted credit card transactions to be preauthorised only, or immediately charged?', '6', 121, 'tep_cfg_select_option(array(\'Standard Payment\', \'Preauthorise\'), ', now());");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Display Errors', 'MODULE_PAYMENT_SECUREPAYXML_ERROR_DISPLAY', 'Yes', 'Do you want to display credit-card errors to customers? If not, they will receive a general \'transaction declined\' message.', '6', 121, 'tep_cfg_select_option(array(\'Yes\', \'No\'), ', now());");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Order Status', 'MODULE_PAYMENT_SECUREPAYXML_ORDER_STATUS_ID', 2, 'When this module is set to Standard Payment mode, which order-status do you want the purchase to be set to?<br />Recommended: <strong>Processing</strong>', '6', 121, 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('PREAUTH Order Status', 'MODULE_PAYMENT_SECUREPAYXML_PREAUTH_ORDER_STATUS_ID', 1, 'When this module is set to Preauthorise mode, which order-status do you want the purchase to be set to?<br />Recommended: <strong>Pending</strong>', '6', 121, 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_SECUREPAYXML_SORT_ORDER', '0', 'Any value greater than zero will cause this payment method to appear in the specified sort order on the checkout-payment page.', '6', 121, now());");
		tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Payment Zone (restrict to)', 'MODULE_PAYMENT_SECUREPAYXML_ZONE', '0', 'If you want only customers from a particular zone to be able to use this payment module, select that zone here.', '6', 121, 'tep_get_zone_class_title', 'tep_cfg_pull_down_zone_classes(', now())");
		
		//Database setup
		$sql = "CREATE TABLE " . TABLE_SECUREPAYXML . " (
			id int(11) unsigned NOT NULL auto_increment,
			customer_id varchar(11) NOT NULL default '',
			banktxnid varchar(7) NOT NULL default '',
			oid int(11) NOT NULL default '0',
			response_code varchar(4) NOT NULL default '999',
			time varchar(25) NOT NULL default '',
			total decimal(15,2) NOT NULL default '0.0000',
			txntype int(3) NOT NULL default '0',
			PRIMARY KEY	(id)
		)";
		tep_db_query($sql);
	}

	function remove()
	{
		@tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key like 'MODULE\_PAYMENT\_SECUREPAYXML\_%'");
		@tep_db_query("drop table " . TABLE_SECUREPAYXML);
	}

	function keys()
	{
		$keys_list = array(
			//'MODULE_PAYMENT_SECUREPAYXML_STATUS',
			'MODULE_PAYMENT_SECUREPAYXML_MERCHANTID',
			'MODULE_PAYMENT_SECUREPAYXML_MERCHANTPASS',
			'MODULE_PAYMENT_SECUREPAYXML_TEST',
			'MODULE_PAYMENT_SECUREPAYXML_MODE',
			'MODULE_PAYMENT_SECUREPAYXML_ERROR_DISPLAY',
			'MODULE_PAYMENT_SECUREPAYXML_ORDER_STATUS_ID',
			'MODULE_PAYMENT_SECUREPAYXML_PREAUTH_ORDER_STATUS_ID',
			'MODULE_PAYMENT_SECUREPAYXML_SORT_ORDER',
			'MODULE_PAYMENT_SECUREPAYXML_ZONE',
		);

		return $keys_list;
	}

	function _log($msg, $suffix = '')
	{
		if(DIR_FS_SQL_CACHE == false)
		{
			return;
		}
		
		$file = $this->_logDir . '/' . 'Securepayxml.log';
		if ($fp = @fopen($file, 'a'))
		{
			@fwrite($fp, "".time().": ".$msg."\n");
			@fclose($fp);
		}
	}
}
?>
